//////////////////////////////////////////////////////////////////////////////
// Filename:          D:\Accelerometer_project\windows/drivers/xps_mems_comm_v1_00_a/src/xps_mems_comm.h
// Version:           1.00.a
// Description:       xps_mems_comm Driver Header File
// Date:              Tue Jun 30 14:35:03 2009 (by Create and Import Peripheral Wizard)
//////////////////////////////////////////////////////////////////////////////

#ifndef XPS_MEMS_COMM_H
#define XPS_MEMS_COMM_H

/***************************** Include Files *******************************/

#include "xbasic_types.h"
#include "xstatus.h"
#include "xio.h"

/************************** Constant Definitions ***************************/


/**
 * User Logic Slave Space Offsets
 * -- SLV_REG0 : user logic slave module register 0
 * -- SLV_REG1 : user logic slave module register 1
 * -- SLV_REG2 : user logic slave module register 2
 * -- SLV_REG3 : user logic slave module register 3
 * -- SLV_REG4 : user logic slave module register 4
 * -- SLV_REG5 : user logic slave module register 5
 * -- SLV_REG6 : user logic slave module register 6
 * -- SLV_REG7 : user logic slave module register 7
 * -- SLV_REG8 : user logic slave module register 8
 * -- SLV_REG9 : user logic slave module register 9
 */
#define XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET (0x00000000)
#define XPS_MEMS_COMM_SLV_REG0_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000000)
#define XPS_MEMS_COMM_SLV_REG1_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000004)
#define XPS_MEMS_COMM_SLV_REG2_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000008)
#define XPS_MEMS_COMM_SLV_REG3_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x0000000C)
#define XPS_MEMS_COMM_SLV_REG4_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000010)
#define XPS_MEMS_COMM_SLV_REG5_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000014)
#define XPS_MEMS_COMM_SLV_REG6_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000018)
#define XPS_MEMS_COMM_SLV_REG7_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x0000001C)
#define XPS_MEMS_COMM_SLV_REG8_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000020)
#define XPS_MEMS_COMM_SLV_REG9_OFFSET (XPS_MEMS_COMM_USER_SLV_SPACE_OFFSET + 0x00000024)

/**
 * Interrupt Controller Space Offsets
 * -- INTR_DISR  : device (peripheral) interrupt status register
 * -- INTR_DIPR  : device (peripheral) interrupt pending register
 * -- INTR_DIER  : device (peripheral) interrupt enable register
 * -- INTR_DIIR  : device (peripheral) interrupt id (priority encoder) register
 * -- INTR_DGIER : device (peripheral) global interrupt enable register
 * -- INTR_ISR   : ip (user logic) interrupt status register
 * -- INTR_IER   : ip (user logic) interrupt enable register
 */
#define XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET (0x00000100)
#define XPS_MEMS_COMM_INTR_DISR_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x00000000)
#define XPS_MEMS_COMM_INTR_DIPR_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x00000004)
#define XPS_MEMS_COMM_INTR_DIER_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x00000008)
#define XPS_MEMS_COMM_INTR_DIIR_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x00000018)
#define XPS_MEMS_COMM_INTR_DGIER_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x0000001C)
#define XPS_MEMS_COMM_INTR_IPISR_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x00000020)
#define XPS_MEMS_COMM_INTR_IPIER_OFFSET (XPS_MEMS_COMM_INTR_CNTRL_SPACE_OFFSET + 0x00000028)

/**
 * Interrupt Controller Masks
 * -- INTR_TERR_MASK : transaction error
 * -- INTR_DPTO_MASK : data phase time-out
 * -- INTR_IPIR_MASK : ip interrupt requeset
 * -- INTR_RFDL_MASK : read packet fifo deadlock interrupt request
 * -- INTR_WFDL_MASK : write packet fifo deadlock interrupt request
 * -- INTR_IID_MASK  : interrupt id
 * -- INTR_GIE_MASK  : global interrupt enable
 * -- INTR_NOPEND    : the DIPR has no pending interrupts
 */
#define INTR_TERR_MASK (0x00000001UL)
#define INTR_DPTO_MASK (0x00000002UL)
#define INTR_IPIR_MASK (0x00000004UL)
#define INTR_RFDL_MASK (0x00000020UL)
#define INTR_WFDL_MASK (0x00000040UL)
#define INTR_IID_MASK (0x000000FFUL)
#define INTR_GIE_MASK (0x80000000UL)
#define INTR_NOPEND (0x80)

/**************************** Type Definitions *****************************/


/***************** Macros (Inline Functions) Definitions *******************/

/**
 *
 * Write a value to a XPS_MEMS_COMM register. A 32 bit write is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is written.
 *
 * @param   BaseAddress is the base address of the XPS_MEMS_COMM device.
 * @param   RegOffset is the register offset from the base to write to.
 * @param   Data is the data written to the register.
 *
 * @return  None.
 *
 * @note
 * C-style signature:
 * 	void XPS_MEMS_COMM_mWriteReg(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Data)
 *
 */
#define XPS_MEMS_COMM_mWriteReg(BaseAddress, RegOffset, Data) \
 	XIo_Out32((BaseAddress) + (RegOffset), (Xuint32)(Data))

/**
 *
 * Read a value from a XPS_MEMS_COMM register. A 32 bit read is performed.
 * If the component is implemented in a smaller width, only the least
 * significant data is read from the register. The most significant data
 * will be read as 0.
 *
 * @param   BaseAddress is the base address of the XPS_MEMS_COMM device.
 * @param   RegOffset is the register offset from the base to write to.
 *
 * @return  Data is the data from the register.
 *
 * @note
 * C-style signature:
 * 	Xuint32 XPS_MEMS_COMM_mReadReg(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
#define XPS_MEMS_COMM_mReadReg(BaseAddress, RegOffset) \
 	XIo_In32((BaseAddress) + (RegOffset))


/**
 *
 * Write/Read 32 bit value to/from XPS_MEMS_COMM user logic slave registers.
 *
 * @param   BaseAddress is the base address of the XPS_MEMS_COMM device.
 * @param   RegOffset is the offset from the slave register to write to or read from.
 * @param   Value is the data written to the register.
 *
 * @return  Data is the data from the user logic slave register.
 *
 * @note
 * C-style signature:
 * 	void XPS_MEMS_COMM_mWriteSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset, Xuint32 Value)
 * 	Xuint32 XPS_MEMS_COMM_mReadSlaveRegn(Xuint32 BaseAddress, unsigned RegOffset)
 *
 */
void XPS_MEMS_COMM_mWriteSlaveReg0(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG0_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg1(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG1_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg2(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG2_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg3(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG3_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg4(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG4_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg5(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG5_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg6(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG6_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg7(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG7_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg8(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG8_OFFSET) + (RegOffset), (Xuint32)(Value));
}
void XPS_MEMS_COMM_mWriteSlaveReg9(Xuint32 BaseAddress,Xuint32 RegOffset, Xuint32 Value) {
 	XIo_Out32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG9_OFFSET) + (RegOffset), (Xuint32)(Value));
}

Xuint32 XPS_MEMS_COMM_mReadSlaveReg0(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG0_OFFSET) + (RegOffset)); 
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg1(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG1_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg2(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG2_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg3(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG3_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg4(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG4_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg5(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG5_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg6(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG6_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg7(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG7_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg8(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG8_OFFSET) + (RegOffset));
}
Xuint32 XPS_MEMS_COMM_mReadSlaveReg9(Xuint32 BaseAddress, Xuint32 RegOffset) {
 	return XIo_In32((BaseAddress) + (XPS_MEMS_COMM_SLV_REG9_OFFSET) + (RegOffset));
}

/************************** Function Prototypes ****************************/


/**
 *
 * Enable all possible interrupts from XPS_MEMS_COMM device.
 *
 * @param   baseaddr_p is the base address of the XPS_MEMS_COMM device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void XPS_MEMS_COMM_EnableInterrupt(void * baseaddr_p);

/**
 *
 * Example interrupt controller handler.
 *
 * @param   baseaddr_p is the base address of the XPS_MEMS_COMM device.
 *
 * @return  None.
 *
 * @note    None.
 *
 */
void XPS_MEMS_COMM_Intr_DefaultHandler(void * baseaddr_p);

/**
 *
 * Run a self-test on the driver/device. Note this may be a destructive test if
 * resets of the device are performed.
 *
 * If the hardware system is not built correctly, this function may never
 * return to the caller.
 *
 * @param   baseaddr_p is the base address of the XPS_MEMS_COMM instance to be worked on.
 *
 * @return
 *
 *    - XST_SUCCESS   if all self-test code passed
 *    - XST_FAILURE   if any self-test code failed
 *
 * @note    Caching must be turned off for this function to work.
 * @note    Self test may fail if data memory and device are not on the same bus.
 *
 */
XStatus XPS_MEMS_COMM_SelfTest(void * baseaddr_p);

#endif // XPS_MEMS_COMM_H
