//----------------------------------------------------------------------------
// user_logic.vhd - module
//----------------------------------------------------------------------------
//
// ***************************************************************************
// ** Copyright (c) 1995-2008 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** Xilinx, Inc.                                                          **
// ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
// ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
// ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
// ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
// ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
// ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
// ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
// ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
// ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
// ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
// ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
// ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
// ** FOR A PARTICULAR PURPOSE.                                             **
// **                                                                       **
// ***************************************************************************
//
//----------------------------------------------------------------------------
// Filename:          user_logic.vhd
// Version:           1.00.a
// Description:       User logic module.
// Date:              Tue Jun 30 14:34:47 2009 (by Create and Import Peripheral Wizard)
// Verilog Standard:  Verilog-2001
//----------------------------------------------------------------------------
// Naming Conventions:
//   active low signals:                    "*_n"
//   clock signals:                         "clk", "clk_div#", "clk_#x"
//   reset signals:                         "rst", "rst_n"
//   generics:                              "C_*"
//   user defined types:                    "*_TYPE"
//   state machine next state:              "*_ns"
//   state machine current state:           "*_cs"
//   combinatorial signals:                 "*_com"
//   pipelined or register delay signals:   "*_d#"
//   counter signals:                       "*cnt*"
//   clock enable signals:                  "*_ce"
//   internal version of output port:       "*_i"
//   device pins:                           "*_pin"
//   ports:                                 "- Names begin with Uppercase"
//   processes:                             "*_PROCESS"
//   component instantiations:              "<ENTITY_>I_<#|FUNC>"
//----------------------------------------------------------------------------

module user_logic
(
  // -- ADD USER PORTS BELOW THIS LINE ---------------
  // --USER ports added here
	//read, 	//this is an input from the software that says to read data from the accelerometer.
	//write, 	//this is an input from the software that says to write data to the accelerometer.

	CE,	//this is the chip enable for the accelerometer
	CCLK,	//this is the chip clock for the accelerometer
	//DATA,  	//this is the data send/receive for the accelerometer
	DATA_I,
	DATA_O,
	DATA_T,
	INT, 	//this is the interrupt pin from the accelerometer
//	debug_out,
	//commState,
  // -- ADD USER PORTS ABOVE THIS LINE ---------------

  // -- DO NOT EDIT BELOW THIS LINE ------------------
  // -- Bus protocol ports, do not add to or delete 
  Bus2IP_Clk,                     // Bus to IP clock
  Bus2IP_Reset,                   // Bus to IP reset
  Bus2IP_Data,                    // Bus to IP data bus
  Bus2IP_BE,                      // Bus to IP byte enables
  Bus2IP_RdCE,                    // Bus to IP read chip enable
  Bus2IP_WrCE,                    // Bus to IP write chip enable
  IP2Bus_Data,                    // IP to Bus data bus
  IP2Bus_RdAck,                   // IP to Bus read transfer acknowledgement
  IP2Bus_WrAck,                   // IP to Bus write transfer acknowledgement
  IP2Bus_Error,                   // IP to Bus error response
  IP2Bus_IntrEvent                // IP to Bus interrupt event
  // -- DO NOT EDIT ABOVE THIS LINE ------------------
); // user_logic

// -- ADD USER PARAMETERS BELOW THIS LINE ------------
// --USER parameters added here 
// -- ADD USER PARAMETERS ABOVE THIS LINE ------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol parameters, do not add to or delete
parameter C_SLV_DWIDTH                   = 32;
parameter C_NUM_REG                      = 10;
parameter C_NUM_INTR                     = 1;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

// -- ADD USER PORTS BELOW THIS LINE -----------------
// --USER ports added here
	//input read; 	//this is an input from the software that says to read data from the accelerometer.
	//input write; 	//this is an input from the software that says to write data to the accelerometer.

	output CE; 	//this is the chip enable for the accelerometer
	output CCLK; 	//this is the chip clock for the accelerometer
	//inout DATA;  	//this is the data send/receive for the accelerometer
	input DATA_I;
	output DATA_O;
	output DATA_T;
	input INT; 	//this is the interrupt pin from the accelerometer 
//	output [91:0]debug_out;
	//output [4:0] commState;
// -- ADD USER PORTS ABOVE THIS LINE -----------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol ports, do not add to or delete
input                                     Bus2IP_Clk;
input                                     Bus2IP_Reset;
input      [0 : C_SLV_DWIDTH-1]           Bus2IP_Data;
input      [0 : C_SLV_DWIDTH/8-1]         Bus2IP_BE;
input      [0 : C_NUM_REG-1]              Bus2IP_RdCE;
input      [0 : C_NUM_REG-1]              Bus2IP_WrCE;
output     [0 : C_SLV_DWIDTH-1]           IP2Bus_Data;
output                                    IP2Bus_RdAck;
output                                    IP2Bus_WrAck;
output                                    IP2Bus_Error;
output     [0 : C_NUM_INTR-1]             IP2Bus_IntrEvent;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

//----------------------------------------------------------------------------
// Implementation
//----------------------------------------------------------------------------

  // --USER nets declarations added here, as needed for user logic

  // Nets for user logic slave model s/w accessible register example
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg0; //NOT USED BECAUSE OF POTENTIAL PROBLEMS WITH ADDRESSING
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg1; //USED AS THE 7 BIT ADDRESS FOR THE ACCELEROMETER FOR MANUAL READ/WRITE CYCLES
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg2; //USED AS THE 8 BIT DATA FOR THE ACCELEROMETER FOR MANUAL WRITE CYCLES
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg3; //USED AS THE FLAG REGISTER. SOFTWARE READ ONLY
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg4; //READ FROM THIS REG TO GET XAXIS DATA
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg5; //READ FROM THIS REG TO GET YAXIS DATA
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg6; //READ FROM THIS REG TO GET ZAXIS DATA
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg7; //READ FROM THIS REG TO GET MANUAL READ DATA
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg8; //WRITES TO THIS REGISTER TRIGGER READ OR WRITE CYCLES write --  32'd0 = read    32'd1 = write
  reg        [0 : C_SLV_DWIDTH-1]           slv_reg9; //used to start automatic polling and control timing. of data slv_reg9[0] == 1'b1 then poll else do nothing
  wire       [0 : 9]                        slv_reg_write_sel;
  wire       [0 : 9]                        slv_reg_read_sel;
  reg        [0 : C_SLV_DWIDTH-1]           slv_ip2bus_data;
  wire                                      slv_read_ack;
  wire                                      slv_write_ack;
  integer                                   byte_index, bit_index;

  // --USER logic implementation added here
// --USER nets declarations added here, as needed for user logic
	reg CE; 	// = 1'b1;
	reg CER; 	// = 1'b0;
	reg CEW;        // = 1'b0;4
	reg CERp;	
	reg CEBlock;   	// = 1'b0;
	reg commDone; 	// = 1'b0;
	reg commProcessing; // = 1'b0; 
	wire CCLK;

	//used for the bidirectional DATA pin	
	reg oe; 	// = 1'b0;
	reg dout; 	// = 1'b0;
	
	//something to note is that the clock signal must not be transitioning if the chip is not in use.
	//ADDITIONALLY the clock signal must be high when the chip enable signal is transitioning.
	
	reg clkEnable;//this one bit register is used as the chip clock.
	
	//Chipscope stuff
//	wire [91:0] debug_out;
	
	
	reg [31:0] FSMClk; //this register is used to timestamp data read from the accelerometer
	reg [31:0] temp_FSM_CLK_Value;
	reg [20:0] timestamp; //this register is used to temporarily store the timestamp for a read
	
	//the following registers hold the 10 bits of acceleration data, the 21 bits of timestamp information, and a new data bit.s
	reg [31:0] xAxis;
	reg [31:0] yAxis;
	reg [31:0] zAxis;
	
	reg [22:0] tempAxisData; //this register is used to hold the updated axis (any of them) while the last read cycle commences

	reg [4:0] commState;
	reg [7:0] dataRead;
	reg [7:0] aAddress;
	reg [7:0] mRead; //register to hold manual read data that is prompted by the software

	
	reg [3:0] axisRead; //a flag to allow this hardware to know if the software has read from the axis data registers
	reg axisReadReset; //used to reset the axisRead value 
	reg commStart; //this register is needed because 3ms for the accelerometer to start up, and 10us after a soft reset.
	reg softReset; //this register is used as a flag to stop the FSM after a soft reset has been issued
	reg commClk; //this register is used so that the communication clock can be controlled by the software.
	//reg clkVar;
	//reg oe_p;
	
	integer i;//used in all for loops
	reg read;
	reg write;

  	assign DATA_O = dout;//(~oe) ? dout : 1'bZ ; //QUESTION: Does this actually properly change or do I need to set oe 1 clock cycle sooner?
	//assign DATA_O = dout; //QUESTION: Does this actually properly change or do I need to set oe 1 clock cycle sooner?
	assign DATA_T = ~oe;//~( oe | oe_p );
	
	assign CCLK = clkEnable ? /*FSMClk[3]*/commClk : 1'b1;//Bus2IP_Clk;
	//assign commClk = slv_reg9[C_SLV_DWIDTH-4] ? Bus2IP_Clk : clkVar;
	//assign commClk = FSMClk[3];

	//assign trigReg = 
//	reg bit9,bit7,state_signal;
	
	reg ackState;
	reg trigAck;
	reg ToutError;
	reg [23:0] timeOut;
/*
	assign debug_out[0] = CE;
	assign debug_out[1] = CCLK;
	assign debug_out[2] = DATA_I;
	assign debug_out[3] = DATA_O;
	assign debug_out[4] = DATA_T;
	assign debug_out[5] = INT;
	assign debug_out[6] = CER;
	assign debug_out[7] = CEW;
	assign debug_out[8] = CERp;
	assign debug_out[9] = CEBlock;
	assign debug_out[10] = commDone;
	assign debug_out[11] = commProcessing;
	assign debug_out[12] = axisReadReset;
	assign debug_out[13] = axisRead[0];
	assign debug_out[14] = axisRead[1];
	assign debug_out[15] = axisRead[2];
	assign debug_out[16] = axisRead[3];
	assign debug_out[17] = commState[0];
	assign debug_out[18] = commState[1];
	assign debug_out[19] = commState[2];
	assign debug_out[20] = commState[3];
	assign debug_out[21] = commState[4];
	assign debug_out[22] = aAddress[0];
	assign debug_out[23] = aAddress[1];
	assign debug_out[24] = aAddress[2];
	assign debug_out[25] = aAddress[3];
	assign debug_out[26] = aAddress[4];
	assign debug_out[27] = aAddress[5];
	assign debug_out[28] = aAddress[6];
	assign debug_out[29] = aAddress[7];
	assign debug_out[30] = timestamp[0];
	assign debug_out[31] = timestamp[1];
	assign debug_out[32] = timestamp[2];
	assign debug_out[33] = timestamp[3];
	assign debug_out[34] = timestamp[4];
	assign debug_out[35] = timestamp[5];
	assign debug_out[36] = timestamp[6];
	assign debug_out[37] = timestamp[7];
	assign debug_out[38] = timestamp[8];
	assign debug_out[39] = timestamp[9];
	assign debug_out[40] = timestamp[10];
	assign debug_out[41] = timestamp[11];
	assign debug_out[42] = timestamp[12];
	assign debug_out[43] = mRead[0];
	assign debug_out[44] = mRead[1];
	assign debug_out[45] = mRead[2];
	assign debug_out[46] = mRead[3];
	assign debug_out[47] = mRead[4];
	assign debug_out[48] = mRead[5];
	assign debug_out[49] = mRead[6];
	assign debug_out[50] = mRead[7];
	assign debug_out[51] = Bus2IP_WrCE[0];
	assign debug_out[52] = Bus2IP_WrCE[1];
	assign debug_out[53] = Bus2IP_WrCE[2];
	assign debug_out[54] = Bus2IP_WrCE[3];
	assign debug_out[55] = Bus2IP_WrCE[4];
	assign debug_out[56] = Bus2IP_WrCE[5];
	assign debug_out[57] = Bus2IP_WrCE[6];
	assign debug_out[58] = Bus2IP_WrCE[7];
	assign debug_out[59] = Bus2IP_WrCE[8];
	assign debug_out[60] = Bus2IP_WrCE[9];
	assign debug_out[61] = Bus2IP_RdCE[0];
	assign debug_out[62] = Bus2IP_RdCE[1];
	assign debug_out[63] = Bus2IP_RdCE[2];
	assign debug_out[64] = Bus2IP_RdCE[3];
	assign debug_out[65] = Bus2IP_RdCE[4];
	assign debug_out[66] = Bus2IP_RdCE[5];
	assign debug_out[67] = Bus2IP_RdCE[6];
	assign debug_out[68] = Bus2IP_RdCE[7];
	assign debug_out[69] = Bus2IP_RdCE[8];
	assign debug_out[70] = Bus2IP_RdCE[9];
	assign debug_out[71] = trigAck;
	assign debug_out[72] = ackState;
	assign debug_out[73] = ToutError;
	assign debug_out[74] = zAxis[9];
	assign debug_out[75] = zAxis[8];
	assign debug_out[76] = zAxis[7];
	assign debug_out[77] = zAxis[6];
	assign debug_out[78] = zAxis[5];
	assign debug_out[79] = zAxis[4];
	assign debug_out[80] = zAxis[3];
	assign debug_out[81] = zAxis[2];
	assign debug_out[82] = slv_read_ack;
	assign debug_out[83] = slv_write_ack;
	assign debug_out[84] = FSMClk[2];
	assign debug_out[85] = FSMClk[1];
	assign debug_out[86] = FSMClk[0];
	assign debug_out[87] = slv_reg9[C_SLV_DWIDTH-1];
	assign debug_out[88] = slv_reg9[0];
	assign debug_out[89] = bit9;
	assign debug_out[90] = bit7;
	assign debug_out[91] = state_signal;
*/

	always@(posedge Bus2IP_Clk) //this always block controls the speed of the FSM
	begin
		if (Bus2IP_Reset == 1)
		begin
			commClk <= 1'b0;
		end
		else
		begin
			case({slv_reg9[C_SLV_DWIDTH-3],slv_reg9[C_SLV_DWIDTH-2]})
			2'b00:
				commClk <= FSMClk[3];
			2'b01:
				commClk <= FSMClk[2];
			2'b10:
				commClk <= FSMClk[1];
			2'b11:
				commClk <= FSMClk[0];
			default:
				commClk <= FSMClk[3];
			endcase
		end
	end
	/**/
	always@(posedge Bus2IP_Clk) //this always block is used to increment the FSMClk.  The FSMClk is used to timestamp the data from the accelerometer.
	begin
		if ( Bus2IP_Reset == 1 )
		begin
			FSMClk <= 0;
			read <= 0;
			write <= 0;
		end
		else
		begin
			if ( (Bus2IP_WrCE[8] & Bus2IP_Data == 32'd0) )
				read <= 1'b1;
			else if (read == 1'b1 && CER == 1'b1)//dont clear the signal until it has been captured.
				read <= 1'b0;
			//read  =  (Bus2IP_WrCE[8] & Bus2IP_Data == 32'd0) ? 1'b1 : 1'b0;
			if ( (Bus2IP_WrCE[8] & Bus2IP_Data == 32'd1) )
				write <= 1'b1;
			else if (write == 1'b1 && CEW == 1'b1)//dont clear the signal until it has been captured.
				write <= 1'b0;
			//write = (Bus2IP_WrCE[8] & Bus2IP_Data == 32'd1) ? 1'b1 : 1'b0;
			
			FSMClk <= FSMClk + 1'b1; //free running counter used for timestamp information
			
			
		end
	end
	
	always@(posedge Bus2IP_Clk or posedge Bus2IP_Reset) //this always block is used to increment the counter.  The counter is used to timestamp the data from the accelerometer.
	begin
		if ( Bus2IP_Reset == 1 )
		begin
			commStart <= 1'b0;
			temp_FSM_CLK_Value <= 32'd0;
			softReset <= 1'b0;
		end
		else
		begin
			if (commStart == 1'b0 && FSMClk == 32'd200000 && softReset == 1'b0) //start the FSM after the power on reset
			begin
				commStart <= 1'b1;
			end
			
			if (slv_reg1 == {0,8'h0A} && slv_reg2[C_SLV_DWIDTH-2] == 1'b1 && commDone == 1'b1 && CEW == 1'b1) //just finished doing a soft reset
			begin
				commStart <= 1'b0;
				temp_FSM_CLK_Value <= FSMClk;
				softReset <= 1'b1;
			end
			
			if ( (FSMClk - temp_FSM_CLK_Value == 1000 || ((32'hFFFFFFFF - temp_FSM_CLK_Value)+ FSMClk) == 1000 ) && softReset == 1'b1) //start the FSM after a soft reset
			begin
				softReset <= 1'b0;
				commStart <= 1'b1;
			end
		end
	end
	
	always@(negedge /*FSMClk[3]*/commClk or posedge Bus2IP_Reset)//this always block is used to control the Chip Enable
	begin
		if ( Bus2IP_Reset == 1 )
		begin
			CE <= 1'b1;
		end
		else if (commStart == 1'b1)
		begin
			if ( (CERp == 1'b1 || CER == 1'b1 || CEW == 1'b1) && (commState == 5'b00000) )
			begin
				CE <= 1'b0;
			end
			
			if (commDone == 1'b1 && clkEnable == 1'b0)
			begin
				CE <= 1'b1;
			end
		end
	end
	
	always@(posedge /*FSMClk[3]*/commClk or posedge Bus2IP_Reset)//this always block is used to control the CCLK output
	begin
		if ( Bus2IP_Reset == 1 )
		begin
			clkEnable <= 1'b0;
		end
		else
		begin
			if (CE == 1'b0 && commDone == 1'b0)
			begin
				clkEnable <= 1'b1;
			end
			
			if (commDone == 1'b1 || CE == 1'b1 || (commState == 5'b10001 || (commState == 5'b10000 /*&& (CER == 1'b1 || CERp == 1'b1)*/)) )
			begin
				clkEnable <= 1'b0;
			end
		end
	end
	
	always@(negedge /*FSMClk[3]*/commClk or posedge Bus2IP_Reset) //this always block is used to start a read or write cycle when prompted by the software OR to perform polling when doing nothing else
	begin
		if ( Bus2IP_Reset == 1 )
        	begin
 			CERp <= 1'b0;
			CER <= 1'b0;
			CEW <= 1'b0;
			CEBlock <= 1'b0;
        	end
		else
		begin
			if ( (read == 1'b0 && write == 1'b0) && CEBlock == 1'b0 && commDone == 1'b0 && slv_reg9[C_SLV_DWIDTH-1] == 1'b1)
			begin
				//need to start implementing the software read cycle
				CERp <= 1'b1;	
				//CEBlock <= 1'b1; //this line is not used
			end
			else if (read == 1'b1 && CEBlock == 1'b0 && commDone == 1'b0)
			begin
				//need to start implementing the software read cycle
				CER <= 1'b1;	
				CEBlock <= 1'b1;
			end
			else if (write == 1'b1 && CEBlock == 1'b0 && commDone == 1'b0)
			begin
				//need to start implementing the write cycle
				CEW <= 1'b1;
				CEBlock <= 1'b1;
			end
			
			if ( (commDone == 1'b1) && (CEBlock == 1'b1 || CERp == 1'b1) )
			begin
				if (CERp == 1'b1 && ( (aAddress == 8'b00000010 || aAddress == 8'b00000100 || aAddress == 8'b00000110) && dataRead[0] == 1'b1 ) ); //dont stop the next polled read as it is the upper 8 bits of the axis we are reading.
				else if(CERp == 1'b1) 
					CERp <= 1'b0;
				
				if(CER == 1'b1 && commProcessing == 1'b0)
				begin
					CER <= 1'b0;
					CEBlock <=1'b0;
				end
				if(CEW == 1'b1 && commProcessing == 1'b0)
				begin
					CEW <= 1'b0;
					CEBlock <= 1'b0;
				end
				
			end
		end
	end
	
	always@(negedge /*FSMClk[3]*/commClk or posedge Bus2IP_Reset)//this always block is for the Read/Write cycles
	begin
		if ( Bus2IP_Reset == 1 )
        	begin
          		commDone <= 1'b0;
			commProcessing <= 1'b0;
         		//CE <= 1'b1;
			aAddress <= 8'b00000010; //02h address is xaxis and the first axis to check for
			commState <= 5'b00000; //need 17 states
			oe <= 1'b0;
			xAxis <= 32'd0;
			yAxis <= 32'd0;
			zAxis <= 32'd0;
        	end
		else if (commStart == 1'b1)
		begin
			if(CERp == 1'b1 && commDone == 1'b0 && CE == 1'b0)//we are doing a polling read cycle (happens when this hardware is not doing something for the software
			begin
				case (commState)
				5'b00000:
					begin
					//CE <= 1'b0;
					oe <= 1'b1;
					dout <= 1'b1; //indicates that this is a read cycle
					commState <= 5'b00001;
					end
				5'b00001:
					begin
					dout <= aAddress[6];
					commState <= 5'b00010;
					end
				5'b00010:
					begin
					dout <= aAddress[5];
					commState <= 5'b00011;
					end
				5'b00011:
					begin
					dout <= aAddress[4];
					commState <= 5'b00100;
					end
				5'b00100:
					begin
					dout <= aAddress[3];
					commState <= 5'b00101;
					end
				5'b00101:
					begin
					dout <= aAddress[2];
					commState <= 5'b00110;
					end
				5'b00110:
					begin
					dout <= aAddress[1];
					commState <= 5'b00111;
					end
				5'b00111:
					begin
					dout <= aAddress[0];
					commState <= 5'b01000;				
					end
				5'b01000:
					begin
					oe <= 1'b0;
					commState <= 5'b01001;
					end
				5'b01001:
					begin
					dataRead[7] <= DATA_I;	
					commState <= 5'b01010;
					end
				5'b01010:
					begin
					dataRead[6] <= DATA_I;			
					commState <= 5'b01011;
					end
				5'b01011:
					begin
					dataRead[5] <= DATA_I;
					commState <= 5'b01100;
					end
				5'b01100:
					begin
					dataRead[4] <= DATA_I;
					commState <= 5'b01101;
					end
				5'b01101:
					begin
					dataRead[3] <= DATA_I;
					commState <= 5'b01110;
					end
				5'b01110:
					begin
					dataRead[2] <= DATA_I;
					commState <= 5'b01111;
					end
				5'b01111:
					begin
					dataRead[1] <= DATA_I;
					commState <= 5'b10000;
					end
				5'b10000:
					begin
					dataRead[0] <= DATA_I;
					for (i=0;i<21;i=i+1)
						timestamp[i] <= FSMClk[i];
					
					commDone <= 1'b1;
					commProcessing <= 1'b1; // indicate on the next clock cycle to process the read information.
					commState <= 5'b00000;
					end
				default:
					commState <= 5'b00000;
				endcase
			end
			else if(CER == 1'b1 && commDone == 1'b0 && CE == 1'b0)//we are doing a software read cycle
			begin
				case (commState)
				5'b00000:
					begin
					//CE <= 1'b0;
					oe <= 1'b1;
					dout <= 1'b1; //indicates that this is a read cycle
					commState <= 5'b00001;
					//state_signal <= 0;			//debug
					end
				5'b00001:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-7];
					commState <= 5'b00010;
					end
				5'b00010:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-6];
					commState <= 5'b00011;
					end
				5'b00011:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-5];
					commState <= 5'b00100;
					end
				5'b00100:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-4];
					commState <= 5'b00101;
					end
				5'b00101:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-3];
					commState <= 5'b00110;
					end
				5'b00110:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-2];
					commState <= 5'b00111;
					end
				5'b00111:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-1];
					commState <= 5'b01000;				
					end
				5'b01000:
					begin
					oe <= 1'b0;
					commState <= 5'b01001;
					end
				5'b01001:
					begin
					mRead[7] <= DATA_I;
					//bit9 <=DATA_I; 					//debug	
					commState <= 5'b01010;
					end
				5'b01010:
					begin
					mRead[6] <= DATA_I;				
					commState <= 5'b01011;
					end
				5'b01011:
					begin
					mRead[5] <= DATA_I;
					//bit7 <= DATA_I;					//debug
					commState <= 5'b01100;
					end
				5'b01100:
					begin
					mRead[4] <= DATA_I;
					commState <= 5'b01101;
					end
				5'b01101:
					begin
					mRead[3] <= DATA_I;
					commState <= 5'b01110;
					end
				5'b01110:
					begin
					mRead[2] <= DATA_I;
					commState <= 5'b01111;
					end
				5'b01111:
					begin
					mRead[1] <= DATA_I;
					commState <= 5'b10000;
					end
				5'b10000:
					begin
					mRead[0] <= DATA_I;
					commState <= 5'b00000;
					commDone <= 1'b1;
					//state_signal <= 1;				//debug
					end
				default:
					commState <= 5'b00000;
				endcase
			end
			else if (CEW == 1'b1 && commDone == 1'b0 && CE == 1'b0)//we are doing a write cycle
			begin
				case (commState)
				5'b00000:
					begin
					//CE <= 1'b0;
					oe <= 1'b1;
					dout <= 1'b0; //indicates that this is a write cycle
					commState <= 5'b00001;
					end
				5'b00001:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-7];
					commState <= 5'b00010;
					end
				5'b00010:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-6];
					commState <= 5'b00011;
					end
				5'b00011:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-5];
					commState <= 5'b00100;
					end
				5'b00100:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-4];
					commState <= 5'b00101;
					end
				5'b00101:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-3];
					commState <= 5'b00110;
					end
				5'b00110:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-2];
					commState <= 5'b00111;
					end
				5'b00111:
					begin
					dout <= slv_reg1[C_SLV_DWIDTH-1];
					commState <= 5'b01000;
					end
				5'b01000:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-8];
					commState <= 5'b01001;
					end
				5'b01001:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-7];
					commState <= 5'b01010;
					end
  				5'b01010:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-6];
					commState <= 5'b01011;
					end
				5'b01011:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-5];
					commState <= 5'b01100;
					end
				5'b01100:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-4];
					commState <= 5'b01101;
					end
				5'b01101:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-3];
					commState <= 5'b01110;
					end
				5'b01110:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-2];
					commState <= 5'b01111;
					end
				5'b01111:
					begin
					dout <= slv_reg2[C_SLV_DWIDTH-1];
					commState <= 5'b10000;
					end
				5'b10000:
					begin
					//CE <= 1'b1;
					oe <= 1'b0;
					commState <= 5'b00000;
					commDone <= 1'b1;
					end
				default:
					commState <= 5'b00000;
				endcase
			end
			else
			begin
				if (commDone == 1'b1)//need to reset the commDone signal
				begin
					commDone <= 1'b0;	
				end
			end
			
			if (commProcessing == 1'b1)//this if statement processes the last polling read performed
			begin
				if(aAddress == 8'b00000010 || aAddress == 8'b00000100 || aAddress == 8'b00000110) //addresses 02h,04h,06h for the new_data bits
				begin
					if( dataRead[0] == 1'b1 )//this is new data
					begin
						//tempAxisData[0] <= 1'b1;//indicator of new data to software
						tempAxisData[0] <= dataRead[6];
						tempAxisData[1] <= dataRead[7];
						for (i=0;i<21;i=i+1)
							tempAxisData[2+i] <= timestamp[i];
						aAddress <= aAddress + 1'b1;
					end
					else //this is not new data so
					begin
						if (aAddress == 8'b00000110)
							aAddress <= 8'b00000010;
						else
							aAddress <= aAddress + 2'b10;
					end
				end
				else if (aAddress == 8'b00000011 || aAddress == 8'b00000101 || aAddress == 8'b00000111) //this data read is from the addresses 03h,05h,07h 
				begin
					if (aAddress == 8'b00000011)
					begin
						xAxis[0] <= 1'b1;//indicator of new data to software
						xAxis[1] <= tempAxisData[0];
						xAxis[2] <= tempAxisData[1];
						for(i=0;i < 8;i=i+1)
							xAxis[3+i] <= dataRead[i];
						for(i=0;i < 21;i=i+1)
							xAxis[11+i] <= tempAxisData[2+i];
						aAddress <= 8'b00000100;
					end
					if (aAddress == 8'b00000101)
					begin
						yAxis[0] <= 1'b1;//indicator of new data to software
						yAxis[1] <= tempAxisData[0];
						yAxis[2] <= tempAxisData[1];
						for(i=0;i < 8;i=i+1)
							yAxis[3+i] <= dataRead[i];
						for(i=0;i < 21;i=i+1)
							yAxis[11+i] <= tempAxisData[2+i];
						aAddress <= 8'b00000110; 	
					end
					if (aAddress == 8'b00000111)
					begin
						zAxis[0] <= 1'b1;//indicator of new data to software
						zAxis[1] <= tempAxisData[0];
						zAxis[2] <= tempAxisData[1];
						for(i=0;i < 8;i=i+1)
							zAxis[3+i] <= dataRead[i];
						for(i=0;i < 21;i=i+1)
							zAxis[11+i] <= tempAxisData[2+i];
						aAddress <= 8'b00000010;
					end
				end 
 
				commProcessing <= 1'b0;
			end

			if (axisRead > 0) //the software has read one of the axis data registers, time to indicate it is no longer new
			begin
				if (axisRead == 4'b0001 && axisReadReset == 1'b0)
					xAxis[0] <= 1'b0;
				if (axisRead == 4'b0010 && axisReadReset == 1'b0)
					yAxis[0] <= 1'b0;
				if (axisRead == 4'b0100 && axisReadReset == 1'b0)
					zAxis[0] <= 1'b0;
				if (axisRead == 4'b1000 && axisReadReset == 1'b0)
					//mRead[0] <= 1'b0;
				axisReadReset <= 1'b1;
			end
			if (axisRead == 0 && axisReadReset == 1'b1)
				axisReadReset <= 1'b0;
		end
	end
  // ------------------------------------------------------
  // Example code to read/write user logic slave model s/w accessible registers
  // 
  // Note:
  // The example code presented here is to show you one way of reading/writing
  // software accessible registers implemented in the user logic slave model.
  // Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  // to one software accessible register by the top level template. For example,
  // if you have four 32 bit software accessible registers in the user logic,
  // you are basically operating on the following memory mapped registers:
  // 
  //    Bus2IP_WrCE/Bus2IP_RdCE   Memory Mapped Register
  //                     "1000"   C_BASEADDR + 0x0
  //                     "0100"   C_BASEADDR + 0x4
  //                     "0010"   C_BASEADDR + 0x8
  //                     "0001"   C_BASEADDR + 0xC
  // 
  // ------------------------------------------------------

  assign
    slv_reg_write_sel = Bus2IP_WrCE[0:9],
    slv_reg_read_sel  = Bus2IP_RdCE[0:9],
    slv_write_ack     = Bus2IP_WrCE[0] || Bus2IP_WrCE[1] || Bus2IP_WrCE[2] || Bus2IP_WrCE[3] || Bus2IP_WrCE[4] || Bus2IP_WrCE[5] || Bus2IP_WrCE[6] || Bus2IP_WrCE[7] || /*Bus2IP_WrCE[8] ||*/ trigAck || Bus2IP_WrCE[9],
    slv_read_ack      = Bus2IP_RdCE[0] || Bus2IP_RdCE[1] || Bus2IP_RdCE[2] || Bus2IP_RdCE[3] || Bus2IP_RdCE[4] || Bus2IP_RdCE[5] || Bus2IP_RdCE[6] || Bus2IP_RdCE[7] || Bus2IP_RdCE[8] || Bus2IP_RdCE[9];

  // implement slave model register(s)
  always @( posedge Bus2IP_Clk )
    begin: SLAVE_REG_WRITE_PROC

      if ( Bus2IP_Reset == 1 )
        begin
          slv_reg0 <= 0;
          slv_reg1 <= 0;
          slv_reg2 <= 0;
          slv_reg3 <= 0;
          slv_reg4 <= 0;
          slv_reg5 <= 0;
          slv_reg6 <= 0;
          slv_reg7 <= 0;
          slv_reg8 <= 0;
          slv_reg9 <= 0;

	  ackState<=0;
	  trigAck<=0;
	  timeOut<=24'd0;
	  ToutError<=0;
        end
      else
/*		slv_reg3[C_SLV_DWIDTH-1] <= commStart; //sync the bit with the value of commStart
		slv_reg3[C_SLV_DWIDTH-2] <= CERp; //sync the bit with the value of CERp
		slv_reg3[C_SLV_DWIDTH-3] <= CER; //sync the bit with the value of CER
		slv_reg3[C_SLV_DWIDTH-4] <= CEW; //sync the bit with the value of CEW
*/
     case(ackState)
	0: begin
	trigAck<=0;
	timeOut<=24'd0;
        case ( slv_reg_write_sel )
          10'b1000000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg0[bit_index] <= Bus2IP_Data[bit_index];
          10'b0100000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg1[bit_index] <= Bus2IP_Data[bit_index];
          10'b0010000000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg2[bit_index] <= Bus2IP_Data[bit_index];
 //          10'b0001000000 :
 //           for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
 //             if ( Bus2IP_BE[byte_index] == 1 )
 //               for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
 //                 slv_reg3[bit_index] <= Bus2IP_Data[bit_index];
          10'b0000100000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg4[bit_index] <= Bus2IP_Data[bit_index];
          10'b0000010000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg5[bit_index] <= Bus2IP_Data[bit_index];
          10'b0000001000 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg6[bit_index] <= Bus2IP_Data[bit_index];
          10'b0000000100 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg7[bit_index] <= Bus2IP_Data[bit_index];
          10'b0000000010 : begin
		ackState<=1;
//            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
//              if ( Bus2IP_BE[byte_index] == 1 )
//                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
//                  slv_reg8[bit_index] <= Bus2IP_Data[bit_index];
	  end
          10'b0000000001 :
            for ( byte_index = 0; byte_index <= (C_SLV_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg9[bit_index] <= Bus2IP_Data[bit_index];
          default : ;
        endcase
	end
	1: begin //wait state
		if(commState != 5'd0) begin
			ackState<=0;
			trigAck<=1;
		end
		else begin
			if(timeOut >= 24'd5000000) begin
				trigAck<=1;
				ackState<=0;
				ToutError<=1;
			end
			else begin
				timeOut<=timeOut+24'd1;
			end
		end
	end
	endcase

    end // SLAVE_REG_WRITE_PROC

  // implement slave model register read mux
  always @( slv_reg_read_sel or slv_reg0 or slv_reg1 or slv_reg2 or slv_reg3 or slv_reg4 or slv_reg5 or slv_reg6 or slv_reg7 or slv_reg8 or slv_reg9 )
    begin: SLAVE_REG_READ_PROC
	  if ( Bus2IP_Reset == 1 )
		begin
			axisRead <= 4'b0000;
		end
      case ( slv_reg_read_sel )
        10'b1000000000 : slv_ip2bus_data <= slv_reg0;
        10'b0100000000 : slv_ip2bus_data <= slv_reg1;
        10'b0010000000 : slv_ip2bus_data <= slv_reg2; 
        10'b0001000000 : slv_ip2bus_data <= {commState[4:0],22'd0,ToutError, CEW, CER, CERp, commStart}; //slv_reg3; 
        10'b0000100000 : begin slv_ip2bus_data <= xAxis; axisRead <= 4'b0001; end//slv_reg4;
        10'b0000010000 : begin slv_ip2bus_data <= yAxis; axisRead <= 4'b0010; end//slv_reg5;
        10'b0000001000 : begin slv_ip2bus_data <= zAxis; axisRead <= 4'b0100; end//slv_reg6;
        10'b0000000100 : begin slv_ip2bus_data <= mRead; axisRead <= 4'b1000; end//slv_reg7;
        10'b0000000010 : slv_ip2bus_data <= slv_reg8;
        10'b0000000001 : slv_ip2bus_data <= slv_reg9;
        default : slv_ip2bus_data <= 0;
      endcase

    end // SLAVE_REG_READ_PROC

  // ------------------------------------------------------------
  // Example code to drive IP to Bus signals
  // ------------------------------------------------------------

  assign IP2Bus_Data    = slv_ip2bus_data;
  assign IP2Bus_WrAck   = slv_write_ack;
  assign IP2Bus_RdAck   = slv_read_ack;
  assign IP2Bus_Error   = 0;

endmodule
